from .classify import ImageClassify, VideoClassify
from .bbox import (
    ImageBoundingBox,
    VideoBoundingBox,
    ImageBoundingBoxEntry,
    VideoBoundingBoxEntry,
)
from .segmentation import ImageSegmentation
